@@ -6,7 +6,6 @@ import re, os.path, codecs
 from Exceptions import *
 
 class IniFile:
-	content = {}
 	defaultGroup = ''
 	fileExtension = ''
 
@@ -19,24 +18,18 @@ class IniFile:
 
 	locale = "(\[([a-zA-Z]+)(_[a-zA-Z]+)?(\.[a-zA-Z\-0-9]+)?(@[a-zA-Z]+)?\])?"
 
-	cache = {}
-
 	def __init__(self):
-		# reset content
 		self.content = dict()
 
 	def parse(self, filename, headers):
 		# for performance reasons
 		content = self.content
 
-		# parse file
-		try:
-			lines = file(filename, 'r').readlines()
-			self.file = filename
-		except IOError:
+		if not os.path.isfile(filename):
 			raise ParsingError("File not found", filename)
 
-		for line in lines:
+		# parse file
+		for line in file(filename,'r'):
 			line = line.strip()
 			# empty line
 			if not line:
@@ -48,17 +41,14 @@ class IniFile:
 			elif line[0] == '[':
 				currentGroup = line.lstrip("[").rstrip("]")
 				if debug and self.hasGroup(currentGroup):
-					raise DuplicateGroupError(currentGroup, self.filename)
+					raise DuplicateGroupError(currentGroup, filename)
 				else:
 					content[currentGroup] = {}
 			# key
 			else:
-				try:
-					tmp = line.split('=',1)
-					key = tmp[0].strip()
-					value = tmp[1].strip()
-				except IndexError:
-					raise ParsingError("Invalid Key=Value pair: %s" % line, filename)
+				index = line.find("=")
+				key = line[0:index].strip()
+				value = line[index+1:].strip()
 				try:
 					if debug and self.hasKey(key, currentGroup):
 						raise DuplicateKeyError(key, currentGroup, filename)
@@ -67,6 +57,8 @@ class IniFile:
 				except IndexError:
 					raise ParsingError("[%s]-Header missing" % headers[0], filename)
 
+		self.file = filename
+
 		# check header
 		for header in headers:
 			if content.has_key(header):
@@ -81,24 +73,17 @@ class IniFile:
 		if not group:
 			group = self.defaultGroup
 
-		cache_name = "".join([self.file,key,group,str(locale),type,str(list)])
-
-		try:
-			return self.cache[cache_name]
-		except KeyError:
-			pass
-
 		# return key (with locale)
-		try:
+		if self.content.has_key(group) and self.content[group].has_key(key):
 			if locale:
 				value = self.content[group][self.__addLocale(key, group)]
 			else:
 				value = self.content[group][key]
-		except KeyError, e:
+		else:
 			if debug:
-				if e == group:
+				if not self.content.has_key(group):
 					raise NoGroupError(group, self.file)
-				else:
+				elif not self.content[group].has_key(key):
 					raise NoKeyError(key, group, self.file)
 			else:
 				if list == True:
@@ -131,8 +116,6 @@ class IniFile:
 			else:
 				result = value
 
-		self.cache[cache_name] = result
-
 		return result
 	# end stuff to access the keys
 
@@ -151,12 +134,11 @@ class IniFile:
 		return list
 
 	def __getBoolean(self, boolean):
-		if boolean == 1:
+		if boolean == 1 or boolean == "true" or boolean == "True":
 			return True
-		elif boolean == 0:
+		elif boolean == 0 or boolean == "false" or boolean == "False":
 			return False
-		else:
-			return boolean.capitalize()
+		return False
 	# end subget
 
 	def resetCache(self):
@@ -392,7 +374,7 @@ class IniFile:
 			value = self.content[group][key]
 			del self.content[group][key]
 			return value
-		except KeyError:
+		except KeyError, e:
 			if debug:
 				if e == group:
 					raise NoGroupError(group, self.file)
