@@ -1,6 +1,6 @@
 """
-Implementation of the XDG Menu Specification Version 0.8
-http://www.freedesktop.org/standards/menu-spec
+Implementation of the XDG Menu Specification Version 0.91
+http://standards.freedesktop.org/menu-spec/
 
 Not Supported (and not planed):
     - <LegacyDir>, <KDELegacyDirs>
@@ -18,6 +18,8 @@ class Menu:
 		# Public stuff
 		self.Name = ""
 		self.Entries = []
+		self.Doc = ""
+		self.Filename = ""
 #		self.Parent = ""
 
 		# Private stuff, only needed for parsing
@@ -103,7 +105,8 @@ class Menu:
 
 	def getEntries(self):
 		for entry in self.Entries:
-			if isinstance(entry, MenuEntry) and entry.Show == True:
+			if isinstance(entry, MenuEntry) \
+			and entry.Show == True:
 				yield entry
 			else:
 				yield entry
@@ -121,7 +124,7 @@ class Menu:
 				for submenu in self.Submenus:
 					submenu.searchEntry(filename, deep, action)
 
-	def getPath(self, org = False):
+	def getPath(self, org = False, toplevel = False):
 		parent = self
 		names=[]
 		while 1:
@@ -135,8 +138,10 @@ class Menu:
 				break
 		names.reverse()
 		path = ""
+		if toplevel == False:
+			names.pop(0)
 		for name in names:
-			path = path + "/" + name
+			path = os.path.join(path, name)
 		return path
 
 	def getName(self):
@@ -368,13 +373,15 @@ class Rule:
 
 	def compile(self):
 		exec("""
-def do(entries, type):
+def do(entries, type, run):
     for entry in entries:
-		if entry.Allocated:
-		    continue
-		if %s:
+		if run == 2 and entry.MatchedInclude == True \
+		or entry.Allocated == True:
+			continue
+		elif %s:
 		    if type == "Include":
 				entry.Add = True
+				entry.MatchedInclude = True
 		    else:
 				entry.Add = False
     return entries
@@ -406,11 +413,11 @@ def do(entries, type):
 
 	def parseFilename(self, value):
 		self.parseNew()
-		self.Rule += "entry.DesktopFileID == '" + value + "'"
+		self.Rule += "entry.DesktopFileID == '%s'" % value.strip()
 
 	def parseCategory(self, value):
 		self.parseNew()
-		self.Rule += "'" + value + "' in entry.Categories"
+		self.Rule += "'%s' in entry.Categories" % value.strip()
 
 	def parseAll(self):
 		self.parseNew()
@@ -454,6 +461,7 @@ class MenuEntry:
 		self.DesktopFileID = Id
 		self.Allocated = Allocated
 		self.Add = False
+		self.MatchedInclude = False
 		# to implement the OnlyShowIn/NotShowIn keys
 		self.Show = True
 		# Caching
@@ -494,7 +502,7 @@ class MenuEntry:
 		else:
 			return False
 
-	def __str_(self):
+	def __str__(self):
 		return self.DesktopFileID
 
 
@@ -520,8 +528,7 @@ tmp["mergeFiles"] = []
 def parse(file = ""):
 	# if no file given, try default files
 	if not file:
-		dirs = xdg_config_dirs
-		for dir in dirs:
+		for dir in xdg_config_dirs:
 			file = os.path.join (dir, "menus" , "applications.menu")
 			if os.path.isdir(dir) and os.path.isfile(file):
 				break
@@ -544,9 +551,10 @@ def parse(file = ""):
 	
 	# parse menufile
 	tmp["Root"] = ""
-	__preparse(doc, file)
-	__parse(doc, tmp["Root"])
+	__parse(doc, file, tmp["Root"])
 	__postparse(tmp["Root"])
+	tmp["Root"].Doc = doc
+	tmp["Root"].Filename = file
 
 	# generate the menu
 	cache = DesktopEntryCache()
@@ -559,62 +567,20 @@ def parse(file = ""):
 
 	return tmp["Root"]
 
-def __preparse(doc, file):
-	# extrace path and basename from file
-	path = os.path.dirname(file)
-	basename = os.path.splitext(os.path.basename(path))[0]
 
-	# replace default dir stuff
-	for name in [ "DefaultAppDirs", "DefaultDirectoryDirs", "DefaultMergeDirs" ]:
-		if name == "DefaultMergeDirs":
-			dirs = xdg_config_dirs
-		else:
-			dirs = xdg_data_dirs
-		dirs.reverse()
-
-		nodes = doc.getElementsByTagName(name)
-		for node in nodes:
-			parent = node.parentNode
-			for dir in dirs:
-				if name == "DefaultAppDirs":
-					value = os.path.join(dir, "applications")
-				elif name == "DefaultDirectoryDirs":
-					value = os.path.join(dir, "desktop-directories")
-				elif name == "DefaultMergeDirs":
-					value = os.path.join(dir, "menus", basename + "-merged")
-
-				entry = doc.createElement(name.replace("Default", "").replace("Dirs", "Dir"))
-				entry.appendChild(doc.createTextNode(value))
-				parent.insertBefore(entry, node)
-			parent.removeChild(node)
-
-	# remove duplicate stuff, convert to absolute path and remove not existing path
-	for name in [ "MergeDir", "MergeFile", "DirectoryDir", "AppDir" ]:
-		values  = []
-		parents = []
-		nodes = doc.getElementsByTagName(name)
-		for node in nodes:
-			if not os.path.isabs(node.childNodes[0].nodeValue):
-				node.childNodes[0].nodeValue = os.path.join(path, node.childNodes[0].nodeValue)
-			value = node.childNodes[0].nodeValue
-			if ( value in values and node.parentNode in parents ) \
-			or ( not os.path.exists(value) ) \
-			or ( name != "MergeFile" and not os.path.isdir(value) ) \
-			or ( name == "MergeFile" and not os.path.isfile(value) ):
-				node.parentNode.removeChild(node)
-			else:
-				parents.append(node.parentNode)
-				values.append(value)
-
-def __parse(node, parent = ""):
+def __parse(node, file, parent = ""):
 	for child in node.childNodes:
 		if child.nodeType == xml.dom.Node.ELEMENT_NODE:
 			if child.tagName == 'Menu':
-				__parseMenu(child, parent)
+				__parseMenu(child, file, parent)
 			elif child.tagName == 'AppDir':
-				parent.addAppDir(child.childNodes[0].nodeValue)
+				__parseAppDir(child.childNodes[0].nodeValue, file, parent)
+			elif child.tagName == 'DefaultAppDirs':
+				__parseDefaultAppDir(file, parent)
 			elif child.tagName == 'DirectoryDir':
-				parent.addDirectoryDir(child.childNodes[0].nodeValue)
+				__parseDirectoryDir(child.childNodes[0].nodeValue, file, parent)
+			elif child.tagName == 'DefaultDirectoryDirs':
+				__parseDefaultDirectoryDir(file, parent)
 			elif child.tagName == 'Name' :
 				parent.Name = child.childNodes[0].nodeValue
 			elif child.tagName == 'Directory' :
@@ -630,13 +596,13 @@ def __parse(node, parent = ""):
 			elif child.tagName == 'Include' or child.tagName == 'Exclude':
 				parent.addRule(Rule(child.tagName, child))
 			elif child.tagName == 'MergeFile':
-				__parseMergeFile(child)
+				__parseMergeFile(child.childNodes[0].nodeValue, child, file, parent)
 			elif child.tagName == 'MergeDir':
-				__parseMergeDir(child)
+				__parseMergeDir(child.childNodes[0].nodeValue, child, file, parent)
+			elif child.tagName == 'DefaultMergeDirs':
+				__parseDefaultMergeDirs(child, file, parent)
 			elif child.tagName == 'Move':
 				parent.addMove(Move(child))
-			elif child.tagName == 'X-Python-MergeEnd':
-				__parseMergeEnd()
 			elif child.tagName == 'Layout':
 				if len(child.childNodes) > 1:
 					parent.Layout = Layout(child)
@@ -683,58 +649,116 @@ def __postparse(menu):
 			submenu.setDeleted(False)
 		if submenu.getOnlyUnallocated() == "notset":
 			submenu.setOnlyUnallocated(False)
-		# Delete <Deleted> menus
-		if submenu.getDeleted() == True:
-			remove.append(submenu)
-			continue
 		# add parent's app/directory dirs
 		for dir in menu.getAppDirs(reverse = True):
 			submenu.addAppDir(dir, 0)
 		for dir in menu.getDirectoryDirs(reverse = True):
 			submenu.addDirectoryDir(dir, 0)
 
+		# get the valid .directory file out of the list
+		entry = ""
+		for directory in submenu.getDirectories():
+			for dir in submenu.getDirectoryDirs():
+				if os.path.exists(os.path.join(dir, directory)):
+					try:
+						entry = DesktopEntry()
+						entry.parse(os.path.join(dir, directory))
+					except:
+						pass
+		if entry:
+			submenu.setDirectory(entry)
+		else:
+			submenu.setDirectory(DesktopEntry())
+
 		# enter submenus
 		__postparse(submenu)
-
-	# get the valid .directory file out of the list
-	entry = ""
-	for directory in menu.getDirectories():
-		for dir in menu.getDirectoryDirs():
-			if os.path.exists(os.path.join(dir, directory)):
-				try:
-					entry = DesktopEntry()
-					entry.parse(os.path.join(dir, directory))
-				except:
-					pass
-	if entry:
-		menu.setDirectory(entry)
-	else:
-		menu.setDirectory(DesktopEntry())
 	
 	# remove menus
 	for submenu in remove:
 		menu.removeSubmenu(submenu)
 
 # Menu parsing stuff
-def __parseMenu(child, parent):
+def __parseMenu(child, file, parent):
 	m = Menu()
-	__parse(child, m)
+	__parse(child, file, m)
 	if parent:
 		parent.addSubmenu(m)
 	else:
 		tmp["Root"] = m
 
+# helper function
+def __check(value, file, type):
+	path = os.path.dirname(file)
+
+	if not os.path.isabs(value):
+		value = os.path.join(path, value)
+
+	if type == "dir" and os.path.exists(value) and os.path.isdir(value):
+		return value
+	elif type == "file" and os.path.exists(value) and os.path.isfile(value):
+		return value
+	else:
+		return False
+
+# App/Directory Dir Stuff
+def __parseAppDir(value, file, parent):
+	value = __check(value, file, "dir")
+	if value:
+		parent.addAppDir(value)
+
+def __parseDefaultAppDir(file, parent):
+	for dir in xdg_data_dirs:
+		__parseAppDir(os.path.join(dir, "applications"), file, parent)
+
+def __parseDirectoryDir(value,file,parent):
+	value = __check(value, file, "dir")
+	if value:
+		parent.addDirectoryDir(value)
+
+def __parseDefaultDirectoryDir(file,parent):
+	for dir in xdg_data_dirs:
+		__parseDirectoryDir(os.path.join(dir, "desktop-directories"), file, parent)
+
 # Merge Stuff
-def __parseMergeFile(child):
-	__mergeFile(child.childNodes[0].nodeValue, child)
+def __parseMergeFile(value, child, file, parent):
+	if child.getAttribute("type") == "parent":
+		for dir in xdg_config_dirs:
+			rel_file = file.replace(dir, "").strip("/")
+			if rel_file != file:
+				for p in xdg_config_dirs:
+					if dir == p:
+						continue
+					if os.path.isfile(os.path.join(p,rel_file)):
+						__mergeFile(os.path.join(p,rel_file),child,parent)
+						break
+	else:
+		value = __check(value, file, "file")
+		if value:
+			__mergeFile(value, child, parent)
 
-def __parseMergeDir(child):
-	files = os.listdir(child.childNodes[0].nodeValue)
-	for file in files:
-		if os.path.splitext(file)[1] == ".menu":
-			__mergeFile(os.path.join(child.childNodes[0].nodeValue, file), child)
+def __parseMergeDir(value, child, file, parent):
+	value = __check(value, file, "dir")
+	if value:
+		files = os.listdir(value)
+		for file in files:
+			if os.path.splitext(file)[1] == ".menu":
+				__mergeFile(os.path.join(value, file), child, parent)
+
+def __parseDefaultMergeDirs(child, file, parent):
+	basename = os.path.splitext(os.path.basename(file))[0]
+	for dir in xdg_config_dirs:
+		__parseMergeDir(os.path.join(dir, "menus", basename + "-merged"), child, file, parent)
+
+def __mergeFile(file, child, parent):
+	# check for infinite loops
+	if file in tmp["mergeFiles"]:
+		if debug:
+			raise ParsingError('Infinite MergeFile loop detected', file)
+		else:
+			pass
+
+	tmp["mergeFiles"].append(file)
 
-def __mergeFile(file, child):
 	# load file
 	try:
 		doc = xml.dom.minidom.parse(file)
@@ -743,26 +767,11 @@ def __mergeFile(file, child):
 	except xml.parsers.expat.ExpatError:
 		raise ParsingError('Not a valid .menu file', file)
 
-	# check for infinite loops
-	if file in tmp["mergeFiles"]:
-		raise ParsingError('Infinite MergeFile loop detected', file)
-
-	tmp["mergeFiles"].append(file)
-
 	# append file
-	__preparse(doc, file)
-	next = child.nextSibling
-	for node in doc.childNodes[1].childNodes:
-		if node.nodeType == xml.dom.Node.ELEMENT_NODE:
-			if node.tagName != "Name":
-				child.parentNode.insertBefore(node, next)
-
-	# Make endnode to pop __mergeFiles again
-	endnode = doc.createElement("X-Python-MergeEnd")
-	child.parentNode.insertBefore(endnode, next)
-
-def __parseMergeEnd():
-	tmp["mergeFiles"].pop()
+	if doc.childNodes[0].nodeType == xml.dom.Node.ELEMENT_NODE:
+		__parse(doc.childNodes[0],file,parent)
+	else:
+		__parse(doc.childNodes[1],file,parent)
 
 # Finally generate the menu
 def __genmenuNotOnlyAllocated(menu, cache):
@@ -770,7 +779,7 @@ def __genmenuNotOnlyAllocated(menu, cache):
 		cache.addEntries(menu.getAppDirs())
 		entries = []
 		for rule in menu.getRules():
-			entries = rule.do(cache.getEntries(menu.getAppDirs()), rule.Type)
+			entries = rule.do(cache.getEntries(menu.getAppDirs()), rule.Type, 1)
 		for entry in entries:
 		    if entry.Add == True:
 				entry.Add = False
@@ -784,7 +793,7 @@ def __genmenuOnlyAllocated(menu, cache):
 		cache.addEntries(menu.getAppDirs())
 		entries = []
 		for rule in menu.getRules():
-			entries = rule.do(cache.getEntries(menu.getAppDirs()), rule.Type)
+			entries = rule.do(cache.getEntries(menu.getAppDirs()), rule.Type, 2)
 		for entry in entries:
 		    if entry.Add == True:
 			#	entry.Add = False
@@ -800,6 +809,13 @@ def sort(menu):
 
 	remove = []
 	for submenu in menu.getSubmenus():
+		# remove menus that should not be displayed
+		if submenu.getDeleted() == True \
+		or submenu.Directory.getHidden() == True \
+		or submenu.Directory.getNoDisplay() == True:
+			remove.append(submenu)
+			continue
+
 		sort(submenu)
 
 		# remove separators at the beginning and at the end
